// -*- C++ -*-
//
// ChColorMap - approximate inverse colour mapping object
//
// Copyright © 1996-1998 Arcane Dragon Software
//                       Chris Herborth (chrish@kagi.com)
//
// Provide an approximate inverse colour mapping into the set of system
// colours.
//
// Creating a ChColorMap object gets the system colours and sets up the
// approximate inverse colour map for those colours.
//
// To find the closest match in the system colour, call getColor() with
// an rgb_color or set of RGB values.  To find the index into the system
// colour map of the closest match, call getIndex().
//
// If the BeOS ever adds the ability to change the system colour map,
// you'll have to call Refresh() when the map changes.
//
// This code is based on a technique described by Tim Kientzle in the
// August 1996 C/C++ User's Journal ("Approximate Inverse Color Mapping").
//
// $Id: ChColorMap.h,v 1.6 1998/04/14 11:39:23 chrish Exp $

#ifndef libCh_COLORMAP_H
#define libCh_COLORMAP_H

#pragma once

#include <interface/InterfaceDefs.h>

struct ChColorMapping {
	unsigned long map;
	unsigned int  index;
};

class ChColorMap {
public:
	ChColorMap();
	~ChColorMap();

	// Refresh the internal colour map; not needed as long as BeOS doesn't
	// let anyone switch the system colour map.
	//
	// Returns B_NO_ERROR if successful, or B_NO_MEMORY if it was unable
	// to allocate memory for the mapping array.
	status_t Refresh( void );

	unsigned int GetIndex( const rgb_color colour ) {
		return search( colour );
	}

	unsigned int GetIndex( const unsigned int r,
						   const unsigned int g,
						   const unsigned int b ) {
		return search( r, g, b );
	}

	rgb_color GetColor( const rgb_color colour ) {
		return _sysColours[search( colour )];
	}

	rgb_color GetColor( const unsigned int r,
					    const unsigned int g,
					    const unsigned int b ) {
		return _sysColours[search( r, g, b )];
	}

private:
	ChColorMap( const ChColorMap& );		// Don't copy me
	ChColorMap& operator=( ChColorMap& );	// Don't assign me

	unsigned long interleave( const rgb_color colour ) {
		return interleave( colour.red, colour.green, colour.blue );
	}

	unsigned long interleave( const unsigned int r,
							  const unsigned int g,
							  const unsigned int b ) {
		return ( _expand[r] << 2 | _expand[g] << 1 | _expand[b] );
	}

	unsigned int search( const rgb_color colour ) {
		return search( colour.red, colour.green, colour.blue );
	}

	unsigned int search( const unsigned int r,
						 const unsigned int g,
						 const unsigned int b );

	unsigned long _expand[256];	// Used by interleave().

	const rgb_color *_sysColours;
	ChColorMapping    *_mapping;

	static char *_rcs_id;
};

#endif
